"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceCollection = void 0;
var Errors_1 = require("../Errors");
var DependencyError_1 = require("../Errors/DependencyError");
var ServiceProvider_1 = require("../ServiceProvider");
var utils_1 = require("../utils");
var mockUtils_1 = require("./mockUtils");
var ServiceCollection = /** @class */ (function () {
    function ServiceCollection() {
        Object.defineProperty(this, "self", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lifetimes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.self = this;
        this.lifetimes = {};
    }
    Object.defineProperty(ServiceCollection.prototype, "addMethod", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (func) {
            return func(this);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "add", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (dependencies) {
            var _this = this;
            var errors = [];
            Object.entries(dependencies).forEach(function (_a) {
                var name = _a[0], data = _a[1];
                if (name in _this.lifetimes)
                    errors.push(new Errors_1.DuplicateDependencyError(name));
                var factory = 'factory' in data
                    ? data.factory
                    // @ts-ignore the factory can be either p => new or (p, props) => new but ts don't know this
                    : function (p, props) { return new data.constructor(p, props); };
                var Lifetime = data.lifetime;
                _this.lifetimes[name] = new Lifetime(name, factory);
            });
            switch (errors.length) {
                case 0:
                    return this;
                case 1:
                    throw errors[0];
                default:
                    throw new DependencyError_1.CollectionDependencyError(errors);
            }
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "get", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            return this.lifetimes[(0, utils_1.extractSelector)(selector)];
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "remove", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            delete this.lifetimes[(0, utils_1.extractSelector)(selector)];
            return this;
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "build", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            var lifetimes = Object.values(this.lifetimes)
                .map(function (e) { return e.clone(); })
                .reduce(function (a, b) {
                a[b.name] = b;
                return a;
            }, {});
            return new ServiceProvider_1.ServiceProvider(lifetimes);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "buildMock", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (mock, defaultMockType) {
            if (mock === void 0) { mock = {}; }
            return (0, mockUtils_1.proxyLifetimes)(this, mock, defaultMockType);
        }
    });
    return ServiceCollection;
}());
exports.ServiceCollection = ServiceCollection;
